// NOTICE: This file is generated by Rollup. To modify it,
// please instead edit the ESM counterpart and rebuild with Rollup (npm run build).
'use strict';

const hasInterpolation = require('./hasInterpolation.cjs');

/**
 * Check whether a keyframes name is standard
 *
 * @param {string} keyframesName
 * @returns {boolean}
 */
function isStandardSyntaxKeyframesName(keyframesName) {
	if (hasInterpolation(keyframesName)) {
		return false;
	}

	return true;
}

module.exports = isStandardSyntaxKeyframesName;
