// NOTICE: This file is generated by Rollup. To modify it,
// please instead edit the ESM counterpart and rebuild with Rollup (npm run build).
'use strict';

const index = require('../formatters/index.cjs');

/**
 * @param {string} separator
 * @param {string} [quote]
 * @returns {string}
 */
function getFormatterOptionsText(separator, quote = '') {
	return Object.keys(index)
		.map((name) => `${quote}${name}${quote}`)
		.join(separator);
}

module.exports = getFormatterOptionsText;
