// NOTICE: This file is generated by Rollup. To modify it,
// please instead edit the ESM counterpart and rebuild with Rollup (npm run build).
'use strict';

const getPreviousNonSharedLineCommentNode = require('./getPreviousNonSharedLineCommentNode.cjs');
const isCustomProperty = require('./isCustomProperty.cjs');
const typeGuards = require('./typeGuards.cjs');
const isStandardSyntaxDeclaration = require('./isStandardSyntaxDeclaration.cjs');

/**
 * @param {import('postcss').Node} node
 * @returns {boolean}
 */
function isAfterStandardPropertyDeclaration(node) {
	const prevNode = getPreviousNonSharedLineCommentNode(node);

	return (
		prevNode !== undefined &&
		typeGuards.isDeclaration(prevNode) &&
		isStandardSyntaxDeclaration(prevNode) &&
		!isCustomProperty(prevNode.prop || '')
	);
}

module.exports = isAfterStandardPropertyDeclaration;
