const HAS_PSV_INTERPOLATION = /\$\(.+?\)/;

/**
 * Check whether a string has postcss-simple-vars interpolation
 *
 * @param {string} string
 * @returns {boolean}
 */
export default function hasPsvInterpolation(string) {
	return HAS_PSV_INTERPOLATION.test(string);
}
