// NOTICE: This file is generated by Rollup. To modify it,
// please instead edit the ESM counterpart and rebuild with Rollup (npm run build).
'use strict';

/**
 * @param {import('postcss').Rule} ruleNode
 * @returns {string}
 */
function getRuleSelector(ruleNode) {
	const raws = ruleNode.raws;

	return (raws.selector && raws.selector.raw) || ruleNode.selector;
}

module.exports = getRuleSelector;
