// NOTICE: This file is generated by Rollup. To modify it,
// please instead edit the ESM counterpart and rebuild with Rollup (npm run build).
'use strict';

/**
 * Check if a string is a single line (i.e. does not contain
 * any newline characters).
 *
 * @param {string} input
 * @returns {boolean}
 */
function isSingleLineString(input) {
	return !/[\n\r]/.test(input);
}

module.exports = isSingleLineString;
