import MurmurHash3 from 'imurmurhash';

/**
 * hash the given string
 * @param {string} str the string to hash
 * @returns {string} the hash
 */
export default function hash(str) {
	return MurmurHash3(str).result().toString(36);
}
