// NOTICE: This file is generated by Rollup. To modify it,
// please instead edit the ESM counterpart and rebuild with Rollup (npm run build).
'use strict';

const keywords = require('../reference/keywords.cjs');

/**
 * Check value is a custom ident
 *
 * @param {string} value
 */
function isCounterResetCustomIdentValue(value) {
	const valueLowerCase = value.toLowerCase();

	if (
		keywords.counterResetKeywords.has(valueLowerCase) ||
		Number.isFinite(Number.parseInt(valueLowerCase, 10))
	) {
		return false;
	}

	return true;
}

module.exports = isCounterResetCustomIdentValue;
