// NOTICE: This file is generated by Rollup. To modify it,
// please instead edit the ESM counterpart and rebuild with Rollup (npm run build).
'use strict';

const isWhitespace = require('./isWhitespace.cjs');

/**
 * Returns a Boolean indicating whether the input string is only whitespace.
 *
 * @param {string} input
 * @returns {boolean}
 */
function isOnlyWhitespace(input) {
	for (const element of input) {
		if (!isWhitespace(element)) {
			return false;
		}
	}

	return true;
}

module.exports = isOnlyWhitespace;
