// NOTICE: This file is generated by Rollup. To modify it,
// please instead edit the ESM counterpart and rebuild with Rollup (npm run build).
'use strict';

const HAS_TPL_INTERPOLATION = /\{.+?\}/s;

/**
 * Check whether a string has JS template literal interpolation or HTML-like template
 *
 * @param {string} string
 * @returns {boolean} If `true`, a string has template literal interpolation
 */
function hasTplInterpolation(string) {
	return HAS_TPL_INTERPOLATION.test(string);
}

module.exports = hasTplInterpolation;
