// NOTICE: This file is generated by Rollup. To modify it,
// please instead edit the ESM counterpart and rebuild with Rollup (npm run build).
'use strict';

const isValidHex = require('./isValidHex.cjs');

/**
 * @param {import('postcss-value-parser').Node} node
 * @returns {boolean}
 */
function isHexColor({ type, value }) {
	return type === 'word' && isValidHex(value);
}

module.exports = isHexColor;
