// NOTICE: This file is generated by Rollup. To modify it,
// please instead edit the ESM counterpart and rebuild with Rollup (npm run build).
'use strict';

const DEFAULT_CACHE_LOCATION = './.stylelintcache';
const CACHE_STRATEGY_METADATA = 'metadata';
const CACHE_STRATEGY_CONTENT = 'content';
const DEFAULT_CACHE_STRATEGY = CACHE_STRATEGY_METADATA;

const DEFAULT_IGNORE_FILENAME = '.stylelintignore';

const DEFAULT_FORMATTER = 'string';

// NOTE: Partially based on `sysexits.h`.
const EXIT_CODE_SUCCESS = 0;
const EXIT_CODE_FATAL_ERROR = 1;
const EXIT_CODE_LINT_PROBLEM = 2;
const EXIT_CODE_INVALID_USAGE = 64;
const EXIT_CODE_INVALID_CONFIG = 78;

const RULE_NAME_ALL = 'all';

const SEVERITY_ERROR = 'error';
const SEVERITY_WARNING = 'warning';
const DEFAULT_SEVERITY = SEVERITY_ERROR;

const DEFAULT_FIX_MODE = 'lax';

exports.CACHE_STRATEGY_CONTENT = CACHE_STRATEGY_CONTENT;
exports.CACHE_STRATEGY_METADATA = CACHE_STRATEGY_METADATA;
exports.DEFAULT_CACHE_LOCATION = DEFAULT_CACHE_LOCATION;
exports.DEFAULT_CACHE_STRATEGY = DEFAULT_CACHE_STRATEGY;
exports.DEFAULT_FIX_MODE = DEFAULT_FIX_MODE;
exports.DEFAULT_FORMATTER = DEFAULT_FORMATTER;
exports.DEFAULT_IGNORE_FILENAME = DEFAULT_IGNORE_FILENAME;
exports.DEFAULT_SEVERITY = DEFAULT_SEVERITY;
exports.EXIT_CODE_FATAL_ERROR = EXIT_CODE_FATAL_ERROR;
exports.EXIT_CODE_INVALID_CONFIG = EXIT_CODE_INVALID_CONFIG;
exports.EXIT_CODE_INVALID_USAGE = EXIT_CODE_INVALID_USAGE;
exports.EXIT_CODE_LINT_PROBLEM = EXIT_CODE_LINT_PROBLEM;
exports.EXIT_CODE_SUCCESS = EXIT_CODE_SUCCESS;
exports.RULE_NAME_ALL = RULE_NAME_ALL;
exports.SEVERITY_ERROR = SEVERITY_ERROR;
exports.SEVERITY_WARNING = SEVERITY_WARNING;
