// NOTICE: This file is generated by Rollup. To modify it,
// please instead edit the ESM counterpart and rebuild with Rollup (npm run build).
'use strict';

/**
 * @type {import('stylelint').PublicApi['createPlugin']}
 */
function createPlugin(ruleName, rule) {
	return {
		ruleName,
		rule,
	};
}

module.exports = createPlugin;
