// NOTICE: This file is generated by Rollup. To modify it,
// please instead edit the ESM counterpart and rebuild with Rollup (npm run build).
'use strict';

/**
 * Check whether a media query is a custom
 * @param {string} mediaQuery
 * @returns {boolean}
 */
function isCustomMediaQuery(mediaQuery) {
	return mediaQuery.startsWith('--');
}

module.exports = isCustomMediaQuery;
