// NOTICE: This file is generated by Rollup. To modify it,
// please instead edit the ESM counterpart and rebuild with Rollup (npm run build).
'use strict';

const HAS_SCSS_INTERPOLATION = /#\{.+?\}/s;

/**
 * Check whether a string has scss interpolation
 *
 * @param {string} string
 * @returns {boolean}
 */
function hasScssInterpolation(string) {
	return HAS_SCSS_INTERPOLATION.test(string);
}

module.exports = hasScssInterpolation;
