// NOTICE: This file is generated by Rollup. To modify it,
// please instead edit the ESM counterpart and rebuild with Rollup (npm run build).
'use strict';

const isSharedLineComment = require('./isSharedLineComment.cjs');

/**
 * @param {import('postcss').Node} node
 * @returns {boolean}
 */
function isAfterComment(node) {
	const previousNode = node.prev();

	if (!previousNode || previousNode.type !== 'comment') {
		return false;
	}

	return !isSharedLineComment(previousNode);
}

module.exports = isAfterComment;
