const HAS_LESS_INTERPOLATION = /@\{.+?\}/;

/**
 * Check whether a string has less interpolation
 *
 * @param {string} string
 * @returns {boolean} If `true`, a string has less interpolation
 */
export default function hasLessInterpolation(string) {
	return HAS_LESS_INTERPOLATION.test(string);
}
