// NOTICE: This file is generated by Rollup. To modify it,
// please instead edit the ESM counterpart and rebuild with Rollup (npm run build).
'use strict';

/**
 * @param {unknown} value
 */
function isNonNegativeInteger(value) {
	return Number.isInteger(value) && typeof value === 'number' && value >= 0;
}

module.exports = isNonNegativeInteger;
