// NOTICE: This file is generated by Rollup. To modify it,
// please instead edit the ESM counterpart and rebuild with Rollup (npm run build).
'use strict';

/**
 * Check if a character is whitespace.
 *
 * @param {string} char
 * @returns {boolean}
 */
function isWhitespace(char) {
	return [' ', '\n', '\t', '\r', '\f'].includes(char);
}

module.exports = isWhitespace;
