// NOTICE: This file is generated by Rollup. To modify it,
// please instead edit the ESM counterpart and rebuild with Rollup (npm run build).
'use strict';

const functions = require('../reference/functions.cjs');

/**
 * Check whether a node is math function
 *
 * @param {import('postcss-value-parser').Node} node postcss-value-parser node
 * @returns {boolean} If `true`, the node is math function
 */
function isMathFunction(node) {
	return node.type === 'function' && functions.mathFunctions.has(node.value.toLowerCase());
}

module.exports = isMathFunction;
