// NOTICE: This file is generated by Rollup. To modify it,
// please instead edit the ESM counterpart and rebuild with Rollup (npm run build).
'use strict';

/**
 * @param {import('postcss').Declaration} decl
 * @returns {string}
 */
function getDeclarationValue(decl) {
	const raws = decl.raws;

	return (raws.value && raws.value.raw) || decl.value;
}

module.exports = getDeclarationValue;
