/**
 * @param {string} source
 * @param {string} [blurChar]
 * @returns {string}
 */
export default function blurInterpolation(source, blurChar = ' ') {
	return source.replace(/[#@{}]+/g, blurChar);
}
