// NOTICE: This file is generated by Rollup. To modify it,
// please instead edit the ESM counterpart and rebuild with Rollup (npm run build).
'use strict';

/**
 * Check if a statement has an block (empty or otherwise).
 *
 * @param {import('postcss').Container} statement
 * @returns {boolean} True if `statement` has a block (empty or otherwise)
 */
function hasBlock(statement) {
	return statement.nodes !== undefined;
}

module.exports = hasBlock;
