// NOTICE: This file is generated by Rollup. To modify it,
// please instead edit the ESM counterpart and rebuild with Rollup (npm run build).
'use strict';

/**
 * Check whether a hex color is standard
 *
 * @param {string} hex
 * @returns {boolean}
 */
function isStandardSyntaxHexColor(hex) {
	// Less map usage (e.g. .myclass { color: #colors[somecolor]; })
	if (hex.includes('[')) {
		return false;
	}

	return true;
}

module.exports = isStandardSyntaxHexColor;
