// NOTICE: This file is generated by Rollup. To modify it,
// please instead edit the ESM counterpart and rebuild with Rollup (npm run build).
'use strict';

/**
 * @param {string} source
 * @param {string} [blurChar]
 * @returns {string}
 */
function blurInterpolation(source, blurChar = ' ') {
	return source.replace(/[#@{}]+/g, blurChar);
}

module.exports = blurInterpolation;
