// NOTICE: This file is generated by Rollup. To modify it,
// please instead edit the ESM counterpart and rebuild with Rollup (npm run build).
'use strict';

const _interopNamespaceDefaultOnly = e => Object.freeze({ __proto__: null, default: e });

/** @type {import('stylelint').PublicApi['rules']} */
const rules = {
	get 'alpha-value-notation'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./alpha-value-notation/index.cjs'))).then((m) => m.default);
	},
	get 'annotation-no-unknown'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./annotation-no-unknown/index.cjs'))).then((m) => m.default);
	},
	get 'at-rule-allowed-list'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./at-rule-allowed-list/index.cjs'))).then((m) => m.default);
	},
	get 'at-rule-descriptor-no-unknown'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./at-rule-descriptor-no-unknown/index.cjs'))).then((m) => m.default);
	},
	get 'at-rule-descriptor-value-no-unknown'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./at-rule-descriptor-value-no-unknown/index.cjs'))).then((m) => m.default);
	},
	get 'at-rule-disallowed-list'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./at-rule-disallowed-list/index.cjs'))).then((m) => m.default);
	},
	get 'at-rule-empty-line-before'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./at-rule-empty-line-before/index.cjs'))).then((m) => m.default);
	},
	get 'at-rule-no-deprecated'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./at-rule-no-deprecated/index.cjs'))).then((m) => m.default);
	},
	get 'at-rule-no-unknown'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./at-rule-no-unknown/index.cjs'))).then((m) => m.default);
	},
	get 'at-rule-no-vendor-prefix'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./at-rule-no-vendor-prefix/index.cjs'))).then((m) => m.default);
	},
	get 'at-rule-prelude-no-invalid'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./at-rule-prelude-no-invalid/index.cjs'))).then((m) => m.default);
	},
	get 'at-rule-property-required-list'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./at-rule-property-required-list/index.cjs'))).then((m) => m.default);
	},
	get 'block-no-empty'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./block-no-empty/index.cjs'))).then((m) => m.default);
	},
	get 'color-function-alias-notation'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./color-function-alias-notation/index.cjs'))).then((m) => m.default);
	},
	get 'color-function-notation'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./color-function-notation/index.cjs'))).then((m) => m.default);
	},
	get 'color-hex-alpha'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./color-hex-alpha/index.cjs'))).then((m) => m.default);
	},
	get 'color-hex-length'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./color-hex-length/index.cjs'))).then((m) => m.default);
	},
	get 'color-named'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./color-named/index.cjs'))).then((m) => m.default);
	},
	get 'color-no-hex'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./color-no-hex/index.cjs'))).then((m) => m.default);
	},
	get 'color-no-invalid-hex'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./color-no-invalid-hex/index.cjs'))).then((m) => m.default);
	},
	get 'comment-empty-line-before'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./comment-empty-line-before/index.cjs'))).then((m) => m.default);
	},
	get 'comment-no-empty'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./comment-no-empty/index.cjs'))).then((m) => m.default);
	},
	get 'comment-pattern'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./comment-pattern/index.cjs'))).then((m) => m.default);
	},
	get 'comment-whitespace-inside'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./comment-whitespace-inside/index.cjs'))).then((m) => m.default);
	},
	get 'comment-word-disallowed-list'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./comment-word-disallowed-list/index.cjs'))).then((m) => m.default);
	},
	get 'container-name-pattern'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./container-name-pattern/index.cjs'))).then((m) => m.default);
	},
	get 'custom-media-pattern'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./custom-media-pattern/index.cjs'))).then((m) => m.default);
	},
	get 'custom-property-empty-line-before'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./custom-property-empty-line-before/index.cjs'))).then((m) => m.default);
	},
	get 'custom-property-no-missing-var-function'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./custom-property-no-missing-var-function/index.cjs'))).then((m) => m.default);
	},
	get 'custom-property-pattern'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./custom-property-pattern/index.cjs'))).then((m) => m.default);
	},
	get 'declaration-block-no-duplicate-custom-properties'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./declaration-block-no-duplicate-custom-properties/index.cjs'))).then(
			(m) => m.default,
		);
	},
	get 'declaration-block-no-duplicate-properties'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./declaration-block-no-duplicate-properties/index.cjs'))).then((m) => m.default);
	},
	get 'declaration-block-no-redundant-longhand-properties'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./declaration-block-no-redundant-longhand-properties/index.cjs'))).then(
			(m) => m.default,
		);
	},
	get 'declaration-block-no-shorthand-property-overrides'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./declaration-block-no-shorthand-property-overrides/index.cjs'))).then(
			(m) => m.default,
		);
	},
	get 'declaration-block-single-line-max-declarations'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./declaration-block-single-line-max-declarations/index.cjs'))).then(
			(m) => m.default,
		);
	},
	get 'declaration-empty-line-before'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./declaration-empty-line-before/index.cjs'))).then((m) => m.default);
	},
	get 'declaration-no-important'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./declaration-no-important/index.cjs'))).then((m) => m.default);
	},
	get 'declaration-property-max-values'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./declaration-property-max-values/index.cjs'))).then((m) => m.default);
	},
	get 'declaration-property-unit-allowed-list'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./declaration-property-unit-allowed-list/index.cjs'))).then((m) => m.default);
	},
	get 'declaration-property-unit-disallowed-list'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./declaration-property-unit-disallowed-list/index.cjs'))).then((m) => m.default);
	},
	get 'declaration-property-value-allowed-list'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./declaration-property-value-allowed-list/index.cjs'))).then((m) => m.default);
	},
	get 'declaration-property-value-disallowed-list'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./declaration-property-value-disallowed-list/index.cjs'))).then((m) => m.default);
	},
	get 'declaration-property-value-keyword-no-deprecated'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./declaration-property-value-keyword-no-deprecated/index.cjs'))).then(
			(m) => m.default,
		);
	},
	get 'declaration-property-value-no-unknown'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./declaration-property-value-no-unknown/index.cjs'))).then((m) => m.default);
	},
	get 'font-family-name-quotes'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./font-family-name-quotes/index.cjs'))).then((m) => m.default);
	},
	get 'font-family-no-duplicate-names'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./font-family-no-duplicate-names/index.cjs'))).then((m) => m.default);
	},
	get 'font-family-no-missing-generic-family-keyword'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./font-family-no-missing-generic-family-keyword/index.cjs'))).then(
			(m) => m.default,
		);
	},
	get 'font-weight-notation'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./font-weight-notation/index.cjs'))).then((m) => m.default);
	},
	get 'function-allowed-list'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./function-allowed-list/index.cjs'))).then((m) => m.default);
	},
	get 'function-calc-no-unspaced-operator'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./function-calc-no-unspaced-operator/index.cjs'))).then((m) => m.default);
	},
	get 'function-disallowed-list'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./function-disallowed-list/index.cjs'))).then((m) => m.default);
	},
	get 'function-linear-gradient-no-nonstandard-direction'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./function-linear-gradient-no-nonstandard-direction/index.cjs'))).then(
			(m) => m.default,
		);
	},
	get 'function-name-case'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./function-name-case/index.cjs'))).then((m) => m.default);
	},
	get 'function-no-unknown'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./function-no-unknown/index.cjs'))).then((m) => m.default);
	},
	get 'function-url-no-scheme-relative'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./function-url-no-scheme-relative/index.cjs'))).then((m) => m.default);
	},
	get 'function-url-quotes'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./function-url-quotes/index.cjs'))).then((m) => m.default);
	},
	get 'function-url-scheme-allowed-list'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./function-url-scheme-allowed-list/index.cjs'))).then((m) => m.default);
	},
	get 'function-url-scheme-disallowed-list'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./function-url-scheme-disallowed-list/index.cjs'))).then((m) => m.default);
	},
	get 'hue-degree-notation'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./hue-degree-notation/index.cjs'))).then((m) => m.default);
	},
	get 'import-notation'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./import-notation/index.cjs'))).then((m) => m.default);
	},
	get 'keyframe-block-no-duplicate-selectors'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./keyframe-block-no-duplicate-selectors/index.cjs'))).then((m) => m.default);
	},
	get 'keyframe-declaration-no-important'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./keyframe-declaration-no-important/index.cjs'))).then((m) => m.default);
	},
	get 'keyframe-selector-notation'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./keyframe-selector-notation/index.cjs'))).then((m) => m.default);
	},
	get 'keyframes-name-pattern'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./keyframes-name-pattern/index.cjs'))).then((m) => m.default);
	},
	get 'layer-name-pattern'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./layer-name-pattern/index.cjs'))).then((m) => m.default);
	},
	get 'length-zero-no-unit'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./length-zero-no-unit/index.cjs'))).then((m) => m.default);
	},
	get 'lightness-notation'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./lightness-notation/index.cjs'))).then((m) => m.default);
	},
	get 'max-nesting-depth'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./max-nesting-depth/index.cjs'))).then((m) => m.default);
	},
	get 'media-feature-name-allowed-list'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./media-feature-name-allowed-list/index.cjs'))).then((m) => m.default);
	},
	get 'media-feature-name-disallowed-list'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./media-feature-name-disallowed-list/index.cjs'))).then((m) => m.default);
	},
	get 'media-feature-name-no-unknown'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./media-feature-name-no-unknown/index.cjs'))).then((m) => m.default);
	},
	get 'media-feature-name-no-vendor-prefix'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./media-feature-name-no-vendor-prefix/index.cjs'))).then((m) => m.default);
	},
	get 'media-feature-name-unit-allowed-list'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./media-feature-name-unit-allowed-list/index.cjs'))).then((m) => m.default);
	},
	get 'media-feature-name-value-allowed-list'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./media-feature-name-value-allowed-list/index.cjs'))).then((m) => m.default);
	},
	get 'media-feature-name-value-no-unknown'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./media-feature-name-value-no-unknown/index.cjs'))).then((m) => m.default);
	},
	get 'media-feature-range-notation'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./media-feature-range-notation/index.cjs'))).then((m) => m.default);
	},
	get 'media-query-no-invalid'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./media-query-no-invalid/index.cjs'))).then((m) => m.default);
	},
	get 'named-grid-areas-no-invalid'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./named-grid-areas-no-invalid/index.cjs'))).then((m) => m.default);
	},
	get 'no-descending-specificity'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./no-descending-specificity/index.cjs'))).then((m) => m.default);
	},
	get 'no-duplicate-at-import-rules'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./no-duplicate-at-import-rules/index.cjs'))).then((m) => m.default);
	},
	get 'no-duplicate-selectors'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./no-duplicate-selectors/index.cjs'))).then((m) => m.default);
	},
	get 'no-empty-source'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./no-empty-source/index.cjs'))).then((m) => m.default);
	},
	get 'no-invalid-double-slash-comments'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./no-invalid-double-slash-comments/index.cjs'))).then((m) => m.default);
	},
	get 'no-invalid-position-at-import-rule'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./no-invalid-position-at-import-rule/index.cjs'))).then((m) => m.default);
	},
	get 'no-irregular-whitespace'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./no-irregular-whitespace/index.cjs'))).then((m) => m.default);
	},
	get 'no-unknown-animations'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./no-unknown-animations/index.cjs'))).then((m) => m.default);
	},
	get 'no-unknown-custom-media'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./no-unknown-custom-media/index.cjs'))).then((m) => m.default);
	},
	get 'no-unknown-custom-properties'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./no-unknown-custom-properties/index.cjs'))).then((m) => m.default);
	},
	get 'number-max-precision'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./number-max-precision/index.cjs'))).then((m) => m.default);
	},
	get 'property-allowed-list'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./property-allowed-list/index.cjs'))).then((m) => m.default);
	},
	get 'property-disallowed-list'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./property-disallowed-list/index.cjs'))).then((m) => m.default);
	},
	get 'property-no-unknown'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./property-no-unknown/index.cjs'))).then((m) => m.default);
	},
	get 'property-no-vendor-prefix'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./property-no-vendor-prefix/index.cjs'))).then((m) => m.default);
	},
	get 'rule-empty-line-before'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./rule-empty-line-before/index.cjs'))).then((m) => m.default);
	},
	get 'rule-selector-property-disallowed-list'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./rule-selector-property-disallowed-list/index.cjs'))).then((m) => m.default);
	},
	get 'selector-anb-no-unmatchable'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./selector-anb-no-unmatchable/index.cjs'))).then((m) => m.default);
	},
	get 'selector-attribute-name-disallowed-list'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./selector-attribute-name-disallowed-list/index.cjs'))).then((m) => m.default);
	},
	get 'selector-attribute-operator-allowed-list'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./selector-attribute-operator-allowed-list/index.cjs'))).then((m) => m.default);
	},
	get 'selector-attribute-operator-disallowed-list'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./selector-attribute-operator-disallowed-list/index.cjs'))).then((m) => m.default);
	},
	get 'selector-attribute-quotes'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./selector-attribute-quotes/index.cjs'))).then((m) => m.default);
	},
	get 'selector-class-pattern'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./selector-class-pattern/index.cjs'))).then((m) => m.default);
	},
	get 'selector-combinator-allowed-list'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./selector-combinator-allowed-list/index.cjs'))).then((m) => m.default);
	},
	get 'selector-combinator-disallowed-list'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./selector-combinator-disallowed-list/index.cjs'))).then((m) => m.default);
	},
	get 'selector-disallowed-list'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./selector-disallowed-list/index.cjs'))).then((m) => m.default);
	},
	get 'selector-id-pattern'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./selector-id-pattern/index.cjs'))).then((m) => m.default);
	},
	get 'selector-max-attribute'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./selector-max-attribute/index.cjs'))).then((m) => m.default);
	},
	get 'selector-max-class'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./selector-max-class/index.cjs'))).then((m) => m.default);
	},
	get 'selector-max-combinators'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./selector-max-combinators/index.cjs'))).then((m) => m.default);
	},
	get 'selector-max-compound-selectors'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./selector-max-compound-selectors/index.cjs'))).then((m) => m.default);
	},
	get 'selector-max-id'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./selector-max-id/index.cjs'))).then((m) => m.default);
	},
	get 'selector-max-pseudo-class'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./selector-max-pseudo-class/index.cjs'))).then((m) => m.default);
	},
	get 'selector-max-specificity'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./selector-max-specificity/index.cjs'))).then((m) => m.default);
	},
	get 'selector-max-type'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./selector-max-type/index.cjs'))).then((m) => m.default);
	},
	get 'selector-max-universal'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./selector-max-universal/index.cjs'))).then((m) => m.default);
	},
	get 'selector-nested-pattern'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./selector-nested-pattern/index.cjs'))).then((m) => m.default);
	},
	get 'selector-no-qualifying-type'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./selector-no-qualifying-type/index.cjs'))).then((m) => m.default);
	},
	get 'selector-no-vendor-prefix'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./selector-no-vendor-prefix/index.cjs'))).then((m) => m.default);
	},
	get 'selector-not-notation'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./selector-not-notation/index.cjs'))).then((m) => m.default);
	},
	get 'selector-pseudo-class-allowed-list'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./selector-pseudo-class-allowed-list/index.cjs'))).then((m) => m.default);
	},
	get 'selector-pseudo-class-disallowed-list'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./selector-pseudo-class-disallowed-list/index.cjs'))).then((m) => m.default);
	},
	get 'selector-pseudo-class-no-unknown'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./selector-pseudo-class-no-unknown/index.cjs'))).then((m) => m.default);
	},
	get 'selector-pseudo-element-allowed-list'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./selector-pseudo-element-allowed-list/index.cjs'))).then((m) => m.default);
	},
	get 'selector-pseudo-element-colon-notation'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./selector-pseudo-element-colon-notation/index.cjs'))).then((m) => m.default);
	},
	get 'selector-pseudo-element-disallowed-list'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./selector-pseudo-element-disallowed-list/index.cjs'))).then((m) => m.default);
	},
	get 'selector-pseudo-element-no-unknown'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./selector-pseudo-element-no-unknown/index.cjs'))).then((m) => m.default);
	},
	get 'selector-type-case'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./selector-type-case/index.cjs'))).then((m) => m.default);
	},
	get 'selector-type-no-unknown'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./selector-type-no-unknown/index.cjs'))).then((m) => m.default);
	},
	get 'shorthand-property-no-redundant-values'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./shorthand-property-no-redundant-values/index.cjs'))).then((m) => m.default);
	},
	get 'string-no-newline'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./string-no-newline/index.cjs'))).then((m) => m.default);
	},
	get 'syntax-string-no-invalid'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./syntax-string-no-invalid/index.cjs'))).then((m) => m.default);
	},
	get 'time-min-milliseconds'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./time-min-milliseconds/index.cjs'))).then((m) => m.default);
	},
	get 'unit-allowed-list'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./unit-allowed-list/index.cjs'))).then((m) => m.default);
	},
	get 'unit-disallowed-list'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./unit-disallowed-list/index.cjs'))).then((m) => m.default);
	},
	get 'unit-no-unknown'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./unit-no-unknown/index.cjs'))).then((m) => m.default);
	},
	get 'value-keyword-case'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./value-keyword-case/index.cjs'))).then((m) => m.default);
	},
	get 'value-no-vendor-prefix'() {
		return Promise.resolve().then(() => /*#__PURE__*/_interopNamespaceDefaultOnly(require('./value-no-vendor-prefix/index.cjs'))).then((m) => m.default);
	},
};

module.exports = rules;
